@@ -44,7 +44,7 @@
 #include <time.h>
 #include <unistd.h>
 
-#ifdef __linux__
+#if defined(__linux__) || (defined(__FreeBSD_kernel__) && defined(__GLIBC__))
 #  include <pty.h>
 #elif defined(__NetBSD__) || defined(__OpenBSD__)
 #  include <util.h>
@@ -54,6 +54,7 @@
 
 #include "einfo.h"
 #include "rc-logger.h"
+#include "queue.h"
 #include "rc.h"
 #include "rc-misc.h"
 
